'use client';
import React, { useState, useEffect, useCallback } from 'react';
import Image from 'next/image';
import {
  DrawerClose,
  DrawerContent,
  DrawerFooter,
  DrawerHeader,
  DrawerTitle,
  Pagination,
  PaginationContent,
  PaginationLink,
  PaginationItem,
  PaginationNext,
  PaginationPrevious,
  ScrollArea,
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
  Button,
} from '@/components/ui';
import {
  getUserHistory,
  getAllHistory,
  type HistoryEntry,
} from '@/lib/hooks';
import { toast } from 'sonner';

type HistoryDrawerProps = {
  user_id: string;
};

export const HistoryDrawer: React.FC<HistoryDrawerProps> = ({ user_id }) => {
  const [history, setHistory] = useState<HistoryEntry[]>([]);
  const [page, setPage] = useState<number>(1);
  const [totalPages, setTotalPages] = useState<number>(1);
  const [totalCount, setTotalCount] = useState<number>(0);
  const [loading, setLoading] = useState<boolean>(true);
  const perPage = 50;

  const fetchHistory = useCallback(async (currentPage: number) => {
    setLoading(true);
    try {
      let response;

      if (user_id && user_id !== '') {
        response = await getUserHistory(user_id, currentPage, perPage);
      } else {
        response = await getAllHistory(currentPage, perPage);
      }

      if (!response.success) {
        throw new Error(response.error);
      }

      setHistory(response.data.data);
      setTotalPages(response.data.meta.total_pages);
      setTotalCount(response.data.meta.total_count);
    } catch (error) {
      const errorMessage = error instanceof Error ? error.message : String(error);
      toast.error(`Error fetching history: ${errorMessage}`);
      setHistory([]);
      setTotalPages(1);
      setTotalCount(0);
    } finally {
      setLoading(false);
    }
  }, [user_id, perPage]);

  useEffect(() => {
    void fetchHistory(page);
  }, [page, fetchHistory]);

  const handlePageChange = (newPage: number) => {
    if (newPage >= 1 && newPage <= totalPages) {
      setPage(newPage);
    }
  };

  const formatTime = (timestamp: string) => {
    const date = new Date(timestamp);
    return date.toLocaleString('en-US', {
      year: 'numeric',
      month: 'short',
      day: 'numeric',
      hour: 'numeric',
      minute: '2-digit',
      hour12: true,
    });
  };

  const getDisplayName = (entry: HistoryEntry) => {
    return entry.user_profile?.full_name || 'Unknown User';
  };

  const getUpdatedByName = (entry: HistoryEntry) => {
    if (!entry.updated_by) return 'Self';
    return entry.updated_by.full_name || 'Unknown';
  };

  return (
    <DrawerContent className='max-w-4xl mx-auto'>
      <DrawerHeader>
        <DrawerTitle>
          <div className='flex flex-row items-center justify-center py-4'>
            <Image
              src='/favicon.png'
              alt='Tech Tracker Logo'
              width={60}
              height={60}
              className='w-8 h-8 md:w-12 md:h-12'
            />
            <h1 className='text-lg md:text-2xl lg:text-4xl font-bold pl-2 md:pl-4'>
              {user_id && user_id !== '' ? 'User History' : 'All History'}
            </h1>
          </div>
          {totalCount > 0 && (
            <p className='text-sm text-muted-foreground text-center'>
              {totalCount} total entries
            </p>
          )}
        </DrawerTitle>
      </DrawerHeader>

      <div className='px-4'>
        <ScrollArea className='h-96 w-full'>
          {loading ? (
            <div className='flex justify-center items-center h-32'>
              <div className='animate-spin rounded-full h-8 w-8 border-b-2 border-primary'></div>
            </div>
          ) : history.length === 0 ? (
            <div className='flex justify-center items-center h-32'>
              <p className='text-muted-foreground'>No history found</p>
            </div>
          ) : (
            <Table>
              <TableHeader>
                <TableRow>
                  <TableHead className='font-semibold'>Name</TableHead>
                  <TableHead className='font-semibold'>Status</TableHead>
                  <TableHead className='font-semibold'>Updated By</TableHead>
                  <TableHead className='font-semibold text-right'>Date & Time</TableHead>
                </TableRow>
              </TableHeader>
              <TableBody>
                {history.map((entry) => (
                  <TableRow key={entry.id}>
                    <TableCell className='font-medium'>
                      {getDisplayName(entry)}
                    </TableCell>
                    <TableCell className='max-w-xs'>
                      <div className='truncate' title={entry.status}>
                        {entry.status}
                      </div>
                    </TableCell>
                    <TableCell className='text-sm text-muted-foreground'>
                      {getUpdatedByName(entry)}
                    </TableCell>
                    <TableCell className='text-right text-sm'>
                      {formatTime(entry.created_at)}
                    </TableCell>
                  </TableRow>
                ))}
              </TableBody>
            </Table>
          )}
        </ScrollArea>
      </div>

      <DrawerFooter>
        {totalPages > 1 && (
          <Pagination>
            <PaginationContent>
              {page > 1 && (
                <PaginationItem>
                  <PaginationPrevious
                    href='#'
                    onClick={(e) => {
                      e.preventDefault();
                      handlePageChange(page - 1);
                    }}
                  />
                </PaginationItem>
              )}

              {totalPages > 10 ? (
                <div className='flex items-center gap-2 text-sm text-muted-foreground'>
                  <span>Page</span>
                  <span className='font-bold text-foreground'>{page}</span>
                  <span>of</span>
                  <span className='font-semibold text-foreground'>{totalPages}</span>
                </div>
              ) : (
                Array.from({ length: totalPages }).map((_, idx) => (
                  <PaginationItem key={idx}>
                    <PaginationLink
                      href='#'
                      isActive={page === idx + 1}
                      onClick={(e) => {
                        e.preventDefault();
                        handlePageChange(idx + 1);
                      }}
                    >
                      {idx + 1}
                    </PaginationLink>
                  </PaginationItem>
                ))
              )}

              {page < totalPages && (
                <PaginationItem>
                  <PaginationNext
                    href='#'
                    onClick={(e) => {
                      e.preventDefault();
                      handlePageChange(page + 1);
                    }}
                  />
                </PaginationItem>
              )}
            </PaginationContent>
          </Pagination>
        )}

        <DrawerClose asChild>
          <Button variant='outline' className='mt-4'>
            Close
          </Button>
        </DrawerClose>
      </DrawerFooter>
    </DrawerContent>
  );
};
