'use server';

import 'server-only';
import { createServerClient } from '@/utils/supabase';
import type { Result } from '.';
import type { Profile } from '@/utils/supabase';

export type UserStatus = Profile & {
  status: string;
  created_at: string;
  updated_by?: Profile;
}

export const getRecentUsers = async (): Promise<Result<string[]>> => {
  try {
    const supabase = await createServerClient();

    // Get users who have had status updates in the last week
    const oneWeekAgo = new Date();
    oneWeekAgo.setDate(oneWeekAgo.getDate() - 7);

    const { data, error } = await supabase
      .from('statuses')
      .select('user_id')
      .gte('created_at', oneWeekAgo.toISOString())
      .order('created_at', { ascending: false });

    if (error) throw error;

    // Get unique user IDs
    const uniqueUserIds = [...new Set(data.map(status => status.user_id))];

    return { success: true, data: uniqueUserIds };
  } catch (error) {
    return {
      success: false,
      error:
        error instanceof Error
          ? error.message
          : 'Unknown error getting recent users',
    };
  }
};

export const getUserStatuses = async (): Promise<Result<UserStatus[]>> => {
  try {
    const supabase = await createServerClient();

    // First get the recent users
    const recentUsersResult = await getRecentUsers();
    if (!recentUsersResult.success) {
      throw new Error(recentUsersResult.error);
    }

    const userIds = recentUsersResult.data;
    if (userIds.length === 0) {
      return { success: true, data: [] };
    }

    // Get the most recent status for each user
    const { data: statusData, error: statusError } = await supabase
      .from('statuses')
      .select('user_id, status, created_at, updated_by_id')
      .in('user_id', userIds)
      .order('created_at', { ascending: false });

    if (statusError) throw statusError;

    if (!statusData) {
      return { success: true, data: [] };
    }

    // Group by user_id and get the most recent status for each user
    const userStatusMap = new Map<string, typeof statusData[0]>();

    statusData.forEach(status => {
      if (!userStatusMap.has(status.user_id)) {
        userStatusMap.set(status.user_id, status);
      }
    });

    // Get all unique user IDs from the status data
    const statusUserIds = Array.from(userStatusMap.keys());

    // Get profile information for these users
    const { data: profileData, error: profileError } = await supabase
      .from('profiles')
      .select('id, full_name, email, avatar_url, provider, updated_at')
      .in('id', statusUserIds);

    if (profileError) throw profileError;

    // Get updated_by profile information
    const updatedByIds = Array.from(userStatusMap.values())
      .map(status => status.updated_by_id)
      .filter((id): id is string => id !== null);

    const { data: updatedByData, error: updatedByError } = await supabase
      .from('profiles')
      .select('id, full_name, email, avatar_url, provider, updated_at')
      .in('id', updatedByIds);

    if (updatedByError) throw updatedByError;

    // Create maps for easy lookup
    const profileMap = new Map(profileData?.map(profile => [profile.id, profile]) ?? []);
    const updatedByMap = new Map(updatedByData?.map(profile => [profile.id, profile]) ?? []);

    // Transform the data to match UserStatus type
    const userStatuses: UserStatus[] = [];

    for (const status of userStatusMap.values()) {
      const profile = profileMap.get(status.user_id);
      const updatedBy = status.updated_by_id ? updatedByMap.get(status.updated_by_id) : undefined;

      if (!profile) continue; // Skip if no profile found

      userStatuses.push({
        // Profile fields
        id: profile.id,
        full_name: profile.full_name,
        email: profile.email,
        avatar_url: profile.avatar_url,
        provider: profile.provider,
        updated_at: profile.updated_at,
        // Status fields
        status: status.status,
        created_at: status.created_at,
        updated_by: updatedBy,
      });
    }
    return { success: true, data: userStatuses };
  } catch (error) {
    return {
      success: false,
      error:
        error instanceof Error
          ? error.message
          : 'Unknown error getting user statuses',
    };
  }
};
